/**********************************************************************
$Id: blockbinaries.c,v 1.2 2002/04/30 06:40:55 bu Exp $

Copyright 2001-2002, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "blockbinaries.h"

#if BLOCK_BINARIES

static regex_t ismessagebinary_base64,ismessagebinary_uuencode;
static int ismessagebinary_compiled=0;

static regex_t groupsbinarymatch_allowed_groups;
static int groupsbinarymatch_compiled=0;

int ismessagebinary(MSGPARTS *mymsg) {

    int err;
#if TESTOUTPUT
    char errbuf[255];
#endif
    
#if TESTOUTPUT
    cflog("ismessagebinary: About to see if regex_t is compiled");
#endif

    if(!ismessagebinary_compiled) {
#if TESTOUTPUT
        cflog("ismessagebinary: About to compile regex_t");
#endif
        if((err=regcomp(&ismessagebinary_base64,
                        "(^[\t>]*[A-Za-z0-9\\+\\/]{59,76}[\t ]*\r?\n){"
                        MAX_ENCODED_LINES "}",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   ))==0) {
#if TESTOUTPUT
	    cflog("ismessagebinary: Compiled ismessagebinary_base64");
#endif	    
        } else {
#if TESTOUTPUT
            regerror(err, &ismessagebinary_base64, &errbuf, 255);
            cflog("ismessagebinary: Error in base64");
#endif
            return 0;
        }

        if((err=regcomp(&ismessagebinary_uuencode,
                        "(^[\t>]*M[\x20-\x60]{60,61}[\t ]*\r?\n){"
                        MAX_ENCODED_LINES "}",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   ))==0) {
#if TESTOUTPUT
	    cflog("ismessagebinary: Compiled ismessagebinary_uuencode");
#endif	    
        } else {
#if TESTOUTPUT
            regerror(err, &ismessagebinary_uuencode, &errbuf, 255);
            cflog("ismessagebinary: Error in uuencode");
#endif
            return 0;
        }
        ismessagebinary_compiled=1;
    }

    if( mymsg->lines > MAX_ENCODED_LINES_I ) {
#if TESTOUTPUT
        cflog("ismessagebinary: More than MAX_ENCODED_LINES, testing base64 regexp");
#endif

        if(!regexec(&ismessagebinary_base64, mymsg->body, 0, NULL, REG_NEWLINE)) {
#if TESTOUTPUT
            cflog("ismessagebinary: Message matches base64 binary");
#endif
            return 1;
        }
                
#if TESTOUTPUT
        cflog("base64 did not match, testing uuencode");
#endif
        if(!regexec(&ismessagebinary_uuencode, mymsg->body, 0, NULL, REG_NEWLINE)) {
#if TESTOUTPUT
            cflog("ismessagebinary: Message matches uuencoded binary");
#endif
            return 1;
        }
        
    }

#if TESTOUTPUT
    cflog("Can not verify that message is a binary message.");
#endif
    return 0;
}

int groupsbinarymatch(char *filtertext) {

    static regex_t groupsbinarymatch_allowed_groups;
    static int groupsbinarymatch_compiled=0;
    char *start,*end;
    char group[255];

#if TESTOUTPUT
    cflog("groupsbinarymatch: About to see if regex_t is compiled");
#endif
    
    if(!groupsbinarymatch_compiled) {
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to compile regex_t");
#endif
        if(!regcomp(&groupsbinarymatch_allowed_groups,
                     "\\.binae?r|^alt\\.sex\\.pictures|^fur\\.artwork"
                     "|multimedia|alt\\.anonymous\\.messages|^de\\.alt\\.dateien"
                     "|^rec\\.games\\.bolo$|^comp\\.security\\.pgp\\.test$"
                     "|sfnet\\.tiedostot",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {
#if TESTOUTPUT
            cflog("groupsbinarymatch: Regexp loaded");
#endif
            groupsbinarymatch_compiled=1;
        } else return -1;    
    }

#if TESTOUTPUT
    cflog("groupsbinarymatch: Regexp is loaded");
#endif
    
    start=filtertext;
    while((end=strchr(start,','))) {
        strncpy(group,start,(end-start));
        group[(end-start-1)]=0;
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to test (inside loop)");
#endif
        if(regexec(&groupsbinarymatch_allowed_groups,group,0,NULL,REG_NEWLINE)) return 1;
	start=end+1;
    }
    
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to test (outside loop)");
#endif

    if(regexec(&groupsbinarymatch_allowed_groups,start,0,NULL,REG_NEWLINE)) return 1;

    return 0;
}

void freeblockbinariesregex() {

    if(ismessagebinary_compiled) {
        regfree(&ismessagebinary_base64);
        regfree(&ismessagebinary_uuencode);
        
        ismessagebinary_compiled=0;
    }
    
    if(groupsbinarymatch_compiled) {
        regfree(&groupsbinarymatch_allowed_groups);
        
        groupsbinarymatch_compiled=0;
    }

}

#endif /* BLOCK_BINARIES */
