/**********************************************************************
$Id: statsuite.c,v 1.7 2002/07/22 18:48:52 bu Exp $

Copyright 2001-2002, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

/**********************************************************
Reject Codes:

  -1: Auto-accepted message.
   0: Filtered & accepted message.
   1: Article exceeded maximum article size.
   2: Article auto-blackholed.
   3: Article has binary data for non-binary group.
   4: Article posted by a spam-bot.
   5: Article exceeded MaxCrossPost
   6: Article duplicates body of another article. 
***********************************************************/
#include "statsuite.h"

#if LOG_STATS

static unsigned long articles=0, autoaccept=0, toobig=0, blackhole=0, bnbg=0;
static unsigned long spambot=0, maxcrossposts=0, duplicate=0;

static unsigned long long autoaccept_size=0, toobig_size=0, blackhole_size=0;
static unsigned long long bnbg_size=0, spambot_size=0, maxcrossposts_size=0;
static unsigned long long duplicate_size=0;

static int begintime=0;
static unsigned long long bytes=0;

int statlog(unsigned long artsize, int code) {

    articles++;
    bytes+=artsize;

    if (articles==1) begintime=(time(NULL)-1);

    switch (code) {
        case -1:   autoaccept++;
                   autoaccept_size+=artsize;
                   break;
        case 1:    toobig++;
                   toobig_size+=artsize;
                   break;        
        case 2:    blackhole++;
                   blackhole_size+=artsize;
                   break;        
        case 3:    bnbg++;
                   bnbg_size+=artsize;
                   break;
        case 4:    spambot++;
                   spambot_size+=artsize;
                   break;
        case 5:    maxcrossposts++;
                   maxcrossposts_size+=artsize;
                   break;                   
        case 6:    duplicate++;
                   duplicate_size+=artsize;
                   break;        
    }

#if 0
    if((articles%1000)==0) statsoutput();
#endif
    
    return 0; 

}

int statsoutput() {

    char *statsfilename=0;
    static FILE *temphandle=NULL;
    int timeused;
    unsigned long long rejbytes=0;
    unsigned long rejcount=0;

    if(!temphandle) {
        if( (statsfilename = getstatsfilename()) != 0 ) {
            temphandle=fopen(statsfilename, "w");
            free(statsfilename);
            if(!temphandle) cflog("Could not open stats logging file... Continuing");
        }
    }

    if(temphandle!=NULL) {
        if(articles>0) {
            timeused=time(NULL) - begintime;
            rejbytes=toobig_size + blackhole_size + bnbg_size + spambot_size + maxcrossposts_size + duplicate_size;
            rejcount=toobig + blackhole + bnbg + spambot + maxcrossposts + duplicate;
            
            fprintf(temphandle, "==============================================================================\n");
            fprintf(temphandle, "Stats Output:\n\n");


            fprintf(temphandle, "Articles: %ld\n", articles);
            fprintf(temphandle, "  Accepted: %ld (%.2f%%)\n", (articles - rejcount), ((float)(articles - rejcount)/articles)*100.0);
            fprintf(temphandle, "    Auto-accepted: %ld (%.2f%%)\n", autoaccept, ((float)autoaccept/articles)*100.0);
            fprintf(temphandle, "  Rejected: %ld (%.2f%%)\n", rejcount, ((float)rejcount/articles)*100.0);
            fprintf(temphandle, "  Kbytes: %llu\n", (bytes/1024) );
            fprintf(temphandle, "    Kbytes Accepted: %llu (%.2f%%)\n", ((bytes-rejbytes)/1024), ((float)(bytes-rejbytes)/bytes)*100.0);
            fprintf(temphandle, "    Kbytes Rejected: %llu (%.2f%%)\n", (rejbytes/1024), ((float)rejbytes/bytes)*100.0 );
            fprintf(temphandle, "Elapsed Time (s): %d\n", timeused);
            fprintf(temphandle, "  Articles/Min: %u\n", (unsigned int)((articles*60)/timeused) );
            fprintf(temphandle, "  KBytes/Min: %lu\n\n", (unsigned long)((bytes/timeused)*(60.0/1024.0)) );

            fprintf(temphandle, "Rejections:\n");
            fprintf(temphandle, "\tMessage Exceeded MaxArticleSize:  \t%ld (%.2f%%)\n", toobig, ((float)toobig/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (toobig_size/1024), ((float)toobig_size/bytes)*100.0);
            fprintf(temphandle, "\tMessage Automatically Blackholed: \t%ld (%.2f%%)\n", blackhole, ((float)blackhole/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (blackhole_size/1024), ((float)blackhole_size/bytes)*100.0);
            fprintf(temphandle, "\tBinary in Non-binary Group:       \t%ld (%.2f%%)\n", bnbg, ((float)bnbg/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (bnbg_size/1024), ((float)bnbg_size/bytes)*100.0);
            fprintf(temphandle, "\tMessage Posted by Spam-Bot:       \t%ld (%.2f%%)\n", spambot, ((float)spambot/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (spambot_size/1024), ((float)spambot_size/bytes)*100.0);
            fprintf(temphandle, "\tMessage has too many crossposts:  \t%ld (%.2f%%)\n", maxcrossposts, ((float)maxcrossposts/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (maxcrossposts_size/1024), ((float)maxcrossposts_size/bytes)*100.0);
            fprintf(temphandle, "\tMessage has Duplicate Body:       \t%ld (%.2f%%)\n", duplicate, ((float)duplicate/articles)*100.0 );
            fprintf(temphandle, "\t        KBytes:                   \t\t%lld (%.2f%%)\n", (duplicate_size/1024), ((float)duplicate_size/bytes)*100.0);
            
            fprintf(temphandle, "==============================================================================\n");
            fflush(temphandle);
        } else cflog("No stats to report.");
    }

    return 0;

}

#endif /* LOG_STATS */