/**********************************************************************
$Id: sumtable.c,v 1.7 2002/04/30 06:40:57 bu Exp $

Copyright 2001-2002, Bulent Yilmaz

Portions Copyright 2002, Michael Heuss

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "sumtable.h"

#if HISTORY_CHECK

struct Node {
    unsigned long long sum;
    int hits;
    struct Node *Left, *Right, *Parent;
};

int tablecheck(unsigned long long checksum) {
    static struct Node *EntireTree=NULL;
    static struct Node *TreeRoot=NULL;
    static long TotalNodes=0, CurrentNode=0;
    struct Node *WorkNode=NULL,*Tracer=NULL;
    int i;
#if TESTOUTPUT
    FILE *temphandle;
#endif

    /* CASE: CHECK TO SEE IF TotalNodes IS INITIALIZED.  IF NOT, DO IT. */
    if(!TotalNodes) TotalNodes=gethistorysize();

    /* CASE: CHECK TO SEE IF THE TREE IS ALLOCATED.  IF NOT, DO IT. */
    if(!EntireTree) {
        /* Request allocation. */
        EntireTree=malloc( TotalNodes * sizeof(struct Node) );
#if TESTOUTPUT
	temphandle=getloghandle();
        fprintf(temphandle, "Malloced %d bytes\n",( (TotalNodes) * sizeof(struct Node) ) );
        fflush(temphandle);
#endif

        /* Initialize the datablock with blank nodes. */
        WorkNode=EntireTree;
        for (i=0;i<TotalNodes;i++) {
#if TESTOUTPUT
		temphandle=getloghandle();
                fprintf(temphandle, "Round %d of %d.  Starting Addy: %p.  Pointer addy: %p.\n",i+1,TotalNodes,(EntireTree),WorkNode);
                fflush(temphandle);
#endif                
                WorkNode->sum=0;
                WorkNode->hits=0;
                WorkNode->Left=NULL;
                WorkNode->Right=NULL;
                WorkNode->Parent=NULL;
                WorkNode+=1;
        }

        /* Since this is the first sample, add it as the Root Node */
#if TESTOUTPUT
        cflog("Adding Root Node.");
#endif
        TreeRoot=WorkNode=EntireTree;
        WorkNode->sum=checksum;
        WorkNode->hits=1;
        CurrentNode=1;

    } else {

        /* Tree is already allocated.  Move to the current node. */

        if (CurrentNode==TotalNodes) CurrentNode=0;
        WorkNode=EntireTree;
        WorkNode+=CurrentNode;

        /* CASE: CHECK TO SEE IF THIS NODE IS CLEAR.  IF NOT, CLEAR IT */

        if (WorkNode->hits) {
                if ((WorkNode->Left==NULL) && (WorkNode->Right==NULL)) {

                        /* SUBCASE: CLEARING CHILDLESS NODE. */
#if TESTOUTPUT
                        cflog("\tNo children.  Fix Parent");
#endif
			if (WorkNode!=TreeRoot) {
	                        if (WorkNode->Parent->Left==WorkNode) WorkNode->Parent->Left=NULL;
	                        else WorkNode->Parent->Right=NULL;
			} else {
				TreeRoot=NULL;
			}
                } else if ((WorkNode->Left!=NULL) && (WorkNode->Right==NULL)) {

                        /* SUBCASE: CLEARING NODE WITH ONLY LEFT CHILD */

#if TESTOUTPUT
			temphandle=getloghandle();
                        fprintf(temphandle, "\t%lld Left Child.  Fix Parent and Left Child.\n",WorkNode->sum);
                        fprintf(temphandle, "WL mem loc: %p, WR mem loc: %p\n",WorkNode->Left,WorkNode->Right);
                        fflush(temphandle);
#endif
                        if (WorkNode!=TreeRoot) {
                                if (WorkNode->Parent->Left==WorkNode) WorkNode->Parent->Left=WorkNode->Left;
                                else WorkNode->Parent->Right=WorkNode->Left;
                        } else {
                                TreeRoot=WorkNode->Left;
                        }
                        WorkNode->Left->Parent=WorkNode->Parent;
                } else if ((WorkNode->Left==NULL) && (WorkNode->Right!=NULL)) {

                        /* SUBCASE: CLEARING NODE WITH ONLY RIGHT CHILD */
#if TESTOUTPUT
                        cflog("\tRight Child.  Fix Parent and Right Child.");
#endif
                        if (WorkNode!=TreeRoot) {
                                if (WorkNode->Parent->Left==WorkNode) WorkNode->Parent->Left=WorkNode->Right;
                                else WorkNode->Parent->Right=WorkNode->Right;
                        } else {
                                TreeRoot=WorkNode->Left;
                        }
                        WorkNode->Right->Parent=WorkNode->Parent;
                } else {
                        /* SUBCASE: CLEARING NODE WITH TWO CHILDREN */
#if TESTOUTPUT
			temphandle=getloghandle();
                        fprintf(temphandle, "\t%lld Has two children.  This is tricky.\n",WorkNode->sum);
                        fflush(temphandle);
#endif

                        /* Check to see which side of the node to favor.
                           This is to attempt better balance. */

                        if ( ((WorkNode->Left->sum)-(WorkNode->sum)) < ((WorkNode->sum)-(WorkNode->Right->sum)) ) {
                                /* Favor the Right. */

                                Tracer=WorkNode->Right;
                                while (Tracer->Left!=NULL) Tracer=Tracer->Left;
#if TESTOUTPUT
				temphandle=getloghandle();
                                fprintf(temphandle, "WL mem loc: %p, WR mem loc: %p, WRL: %p\n",WorkNode->Left,WorkNode->Right,WorkNode->Right->Left);
                                fprintf(temphandle, "WL mem loc: %p, TPL: %p, TR: %p\n",WorkNode->Left,Tracer->Parent->Left, Tracer->Right); 
                                fflush(temphandle);
#endif
                                if (Tracer->Parent->Left==Tracer) Tracer->Parent->Left=Tracer->Right;
                                else Tracer->Parent->Right=Tracer->Right;
                                if (Tracer->Right!=NULL) Tracer->Right->Parent=Tracer->Parent;
                        } else {
                                /* Favor the Left. */

                                Tracer=WorkNode->Left;
                                while (Tracer->Right!=NULL) Tracer=Tracer->Right;
                                if (Tracer->Parent->Left==Tracer) Tracer->Parent->Left=Tracer->Left;
                                else Tracer->Parent->Right=Tracer->Left;
                                if (Tracer->Left!=NULL) Tracer->Left->Parent=Tracer->Parent;
                        }
                        if (WorkNode->Left) WorkNode->Left->Parent=Tracer;
                        if (WorkNode->Right) WorkNode->Right->Parent=Tracer;
                        Tracer->Left=WorkNode->Left;
                        Tracer->Right=WorkNode->Right;
                        if (WorkNode!=TreeRoot) {
                                if (WorkNode->Parent->Left==WorkNode) WorkNode->Parent->Left=Tracer;
                                else WorkNode->Parent->Right=Tracer;
                                Tracer->Parent=WorkNode->Parent;
                        } else {
#if TESTOUTPUT
                                cflog("New Root");
#endif
                                TreeRoot=Tracer;
                                Tracer->Parent=NULL;
                        }
                        
                }

                /* CLEAR WORKNODE */
                WorkNode->sum=0;
                WorkNode->hits=0;
                WorkNode->Left=NULL;
                WorkNode->Right=NULL;
                WorkNode->Parent=NULL;
        }
        /* END CASE: CHECK TO SEE IF THIS NODE IS CLEAR.  IF NOT, CLEAR IT */

        /* CASE: ADD SUM TO TREE.  IF IT IS ALREADY THERE, UPDATE IT.  ASSUME 
           NODE IS CLEAR. */

        WorkNode->sum=checksum;
        WorkNode->hits=1;
        for (Tracer=TreeRoot;;) {
	    if (Tracer==NULL) {
		/* SUBCASE: Tracer is NULL because TreeRoot is NULL - thanks Bu */

		TreeRoot=WorkNode;
		CurrentNode++;
		return 1;
	    }
#if TESTOUTPUT
            temphandle=getloghandle();
            fprintf(temphandle, "\t\tStepping on %lld ",Tracer->sum);
            fflush(temphandle);
#endif
            if(checksum>Tracer->sum) {
                /* SUBCASE: MATCH NOT FOUND.  MOVING RIGHT OR ADDING RIGHT. */

                if (Tracer->Right==NULL) { 
                    Tracer->Right=WorkNode; 
                    WorkNode->Parent=Tracer;
#if TESTOUTPUT
                    temphandle=getloghandle();
                    fprintf(temphandle, "\tAdded Right of %lld\n\n",Tracer->sum); 
                    fflush(temphandle);
#endif
                    CurrentNode++;
                    return 1; 
                }
#if TESTOUTPUT
                cflog("Going Right");
#endif
                Tracer=Tracer->Right;
            } else if (checksum<Tracer->sum) {
                /* SUBCASE: MATCH NOT FOUND.  MOVING LEFT OR ADDING LEFT. */

                if (Tracer->Left==NULL) { 
                    Tracer->Left=WorkNode; 
                    WorkNode->Parent=Tracer;
#if TESTOUTPUT
                    temphandle=getloghandle();
                    fprintf(temphandle, "\tAdded Left of %lld\n\n",Tracer->sum);
                    fflush(temphandle);
#endif
                    CurrentNode++;
                    return 1; 
                }
#if TESTOUTPUT
                cflog("Going Left");
#endif
                Tracer=Tracer->Left;
            } else {
                /* SUBCASE: MATCH FOUND.  POPING TO FRONT. */

                WorkNode->hits=Tracer->hits+1;
                WorkNode->Left=Tracer->Left;
                WorkNode->Right=Tracer->Right;
                WorkNode->Parent=Tracer->Parent;
                if (Tracer->Left) Tracer->Left->Parent=WorkNode;
                if (Tracer->Right) Tracer->Right->Parent=WorkNode;
                if (Tracer->Parent) {
                        if (Tracer->Parent->Left==Tracer) Tracer->Parent->Left=WorkNode;
                        else Tracer->Parent->Right=WorkNode;
                } else {
                        TreeRoot=WorkNode;
                }
                Tracer->sum=0;
                Tracer->hits=0;
                Tracer->Left=NULL;
                Tracer->Right=NULL;
                Tracer->Parent=NULL;
#if TESTOUTPUT
                cflog("Spam Suspect.");
#endif
                return 0;
            }
        }
    /* END CASE: SUM DOES NOT EXIST IN TREE.  ADD IT. */

    }
    /* We should only get here once. */

    return 1;
}


long long calc_sum64(char *instring, long csize) {

    long long sum=0;
    long long *nextptr=NULL;
    long long nextnum;
    double maxsize;
    
    unsigned long int i;
    
    nextptr=(long long *)instring;
    
    if(nextptr) {
        /* Modification.  Make sure we never sum more than 64K of data.
           mheuss 02/07/02 */
	if ((ceil(csize/8)) > (8000.0)) maxsize=8000.0;
	else maxsize=ceil(csize/8);
//        for(i=0; i < (ceil(csize/8)); i++) {
        for(i=0; i < (maxsize); i++) {
        
            nextnum = (*(nextptr+i));
        
            sum=sum +  ( (nextnum << (sum & 63)) | (nextnum >> ( 64 - (sum & 63) )) );
        }
    }
#if TESTOUTPUT
    else cflog("Pointer did not cast properly.");
#endif
    
    return sum;
}

int summatch(char *instr, long slen) {
    
#if TESTOUTPUT
    FILE *loghandle;
#endif
    long long sum;
    char *alignstr;

#if __SOLARIS2__
    alignstr=memalign( 8, ((ceil(slen/8)+1)*8) );
#else
    alignstr=malloc( ((ceil(slen/8)+1)*8) );
#endif

    memcpy(alignstr, instr, slen);

#if TESTOUTPUT
    cflog("summatch: About to determine article body size");
#endif
    
#if TESTOUTPUT
    loghandle=getloghandle();
    fprintf(loghandle, "summatch: Processing article body.  Size: %ld.\n", slen);
    fflush(loghandle);
#endif
    
    sum=calc_sum64(alignstr, slen);

#if TESTOUTPUT
    loghandle=getloghandle();
    fprintf(loghandle, "summatch: Sum value: %llx.\n", sum);
    fflush(loghandle);
#endif

    free(alignstr);
    
    if(sum) {
        return tablecheck(sum);
    } else return 1;
    
}
#endif /* HISTORY_CHECK */
